@@ -72,7 +72,11 @@ static void gff_load_metrics (GnomeFontFace *face, gint glyph);
 static void gff_load_outline (GnomeFontFace *face, gint glyph);
 
 static void gf_pso_print_sized (GnomeFontPsObject *pso, const guchar *text, gint size);
-static void gf_pso_sprintf (GnomeFontPsObject * pso, const gchar * format, ...);
+static void gf_pso_sprintf (GnomeFontPsObject * pso, 
+			    const gchar * format, ...);
+static gint gf_pso_print_double (GnomeFontPsObject *pso, const gchar *format,
+				 gdouble x);
+
 static void gnome_font_face_ps_embed_ensure_size (GnomeFontPsObject * pso, gint size);
 
 #define GFE_IS_T1(e) ((e)->type == GP_FONT_ENTRY_TYPE1)
@@ -212,7 +216,8 @@ gnome_font_face_get_prop (GObject *o, guint id, GValue *value, GParamSpec *pspec
 		g_value_set_string (value, face->entry->familyname);
 		break;
 	case PROP_WEIGHT:
-		g_value_set_string (value, face->entry->weight);
+		/* FIXME: we should be using the GnomeFontWeight Weight */
+		g_value_set_string (value, face->entry->speciesname);
 		break;
 	case PROP_ITALICANGLE:
 		/* FIXME: implement (Lauris) */
@@ -537,34 +542,35 @@ gnome_font_face_find (const guchar *name)
 	}
 }
 
-/* fixme: Do some well-known substitutions here */
-
 GnomeFontFace *
 gnome_font_face_find_closest (const guchar *name)
 {
-	GnomeFontFace *face;
-	GPFontMap *map;
-
-	face = NULL;
-
-	map = gp_fontmap_get ();
+	GnomeFontFace *face = NULL;
 
 	if (name)
 		face = gnome_font_face_find (name);
 
-	if (!face && map->fonts) {
-		/* No face, no default, load whatever font is first */
-		GPFontEntry *e;
-		e = (GPFontEntry *) map->fonts->data;
-		if (e->face) {
-			gnome_font_face_ref (e->face);
-		} else {
-			gff_face_from_entry (e);
+	if (!face)
+		face = gnome_font_face_find ("Sans Regular");
+
+	if (!face) {
+		GPFontMap *map = gp_fontmap_get ();
+		if (map && map->fonts) {
+			/* No face, no default, load whatever font is first */
+			GPFontEntry *e;
+			e = (GPFontEntry *) map->fonts->data;
+			if (e->face) {
+				gnome_font_face_ref (e->face);
+			} else {
+				gff_face_from_entry (e);
+			}
+			face = e->face;
 		}
-		face = e->face;
+		
+		gp_fontmap_release (map);
 	}
 
-	gp_fontmap_release (map);
+	g_return_val_if_fail (face != NULL, NULL);
 
 	return face;
 }
@@ -668,12 +674,42 @@ gnome_font_face_find_closest_from_pango_description (const PangoFontDescription
 
 	g_return_val_if_fail (desc != NULL, NULL);
 
-	weight = (pango_font_description_get_weight (desc) + 49)/ 100;
+	/* pango_font_description_get_weight returns a numerical enum */
+	/* value in the range from 100 to 900 with                    */
+	/* PANGO_WEIGHT_ULTRALIGHT = 200                              */
+	/* PANGO_WEIGHT_NORMAL     = 400                              */
+        /* PANGO_WEIGHT_HEAVY      = 900                              */
+
+	weight = pango_font_description_get_weight (desc);
 	style = pango_font_description_get_style (desc);
-	italic = ((style == PANGO_STYLE_OBLIQUE) || (style == PANGO_STYLE_ITALIC));
+	italic = ((style == PANGO_STYLE_OBLIQUE) 
+		  || (style == PANGO_STYLE_ITALIC));
 	family_name = pango_font_description_get_family (desc);
 
-	return gnome_font_face_find_closest_from_weight_slant (family_name, weight, italic);
+	/* gnome_font_face_find_closest_from_weight_slant requires as */
+	/* weight a GnomeFontWeight, a numerical value ranging from   */
+	/* 100 to 1100 with                                           */
+        /* 	GNOME_FONT_LIGHTEST = 100, */
+        /* 	GNOME_FONT_EXTRA_LIGHT = 100, */
+        /* 	GNOME_FONT_THIN = 200, */
+        /* 	GNOME_FONT_LIGHT = 300, */
+        /* 	GNOME_FONT_BOOK = 400, */
+        /* 	GNOME_FONT_REGULAR = 400, */
+        /* 	GNOME_FONT_MEDIUM = 500, */
+        /* 	GNOME_FONT_SEMI = 600, */
+        /* 	GNOME_FONT_DEMI = 600, */
+        /* 	GNOME_FONT_BOLD = 700, */
+        /* 	GNOME_FONT_HEAVY = 900, */
+        /* 	GNOME_FONT_EXTRABOLD = 900, */
+        /* 	GNOME_FONT_BLACK = 1000, */
+        /* 	GNOME_FONT_EXTRABLACK = 1100, */
+        /* 	GNOME_FONT_HEAVIEST = 1100 */
+
+	/* Since these values are reasonably close, we retain the same */
+        /* numerical values. */
+
+	return gnome_font_face_find_closest_from_weight_slant 
+		(family_name, weight, italic);
 }
 
 /* This returns GList of (guchar *) */
@@ -787,7 +823,7 @@ gff_load (GnomeFontFace *face)
 		entry = face->entry;
 	}
 
-	ft_result = FT_New_Face (ft_library, entry->file, 0, &ft_face);
+	ft_result = FT_New_Face (ft_library, entry->file, entry->index, &ft_face);
 	g_return_val_if_fail (ft_result == FT_Err_Ok, FALSE);
 
 	psname = FT_Get_Postscript_Name (ft_face);
@@ -893,7 +929,9 @@ gff_load_outline (GnomeFontFace *face, gint glyph)
 	g_assert (face->ft_face);
 	g_assert (!face->glyphs[glyph].bpath);
 
-	FT_Load_Glyph (face->ft_face, glyph, FT_LOAD_NO_SCALE | FT_LOAD_NO_HINTING | FT_LOAD_NO_BITMAP);
+	FT_Load_Glyph (face->ft_face, glyph,
+		       FT_LOAD_NO_SCALE | FT_LOAD_NO_HINTING |
+		       FT_LOAD_NO_BITMAP | FT_LOAD_IGNORE_TRANSFORM);
 
 	a[0] = a[3] = face->ft2ps;
 	a[1] = a[2] = a[4] = a[5] = 0.0;
@@ -1043,7 +1081,7 @@ static void gnome_font_face_ps_embed_ensure_size (GnomeFontPsObject *pso, gint s
  */
 
 GnomeFontPsObject *
-gnome_font_face_pso_new (GnomeFontFace *face, guchar *residentname)
+gnome_font_face_pso_new (GnomeFontFace *face, guchar *residentname, gint instance)
 {
 	GnomeFontPsObject *pso;
 
@@ -1056,7 +1094,10 @@ gnome_font_face_pso_new (GnomeFontFace *face, guchar *residentname)
 	g_object_ref (G_OBJECT (face));
 	if (residentname)
 		pso->residentname = g_strdup (residentname);
-	pso->encodedname = g_strdup_printf ("GnomeUni-%s", face->psname);
+	if (instance == 0)
+		pso->encodedname = g_strdup_printf ("GnomeUni-%s", face->psname);
+	else
+		pso->encodedname = g_strdup_printf ("GnomeUni-%s_%03d", face->psname, instance);
 	pso->bufsize = 0;
 	pso->length = 0;
 	pso->buf = NULL;
@@ -1217,7 +1258,7 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
 	if (pso->encodedbytes == 1) {
 		gint glyph;
 		/* 8-bit vector */
-		gf_pso_sprintf (pso, "/%s findfont dup length dict begin\n", embeddedname);
+		gf_pso_sprintf (pso, "(%s) cvn findfont dup length dict begin\n", embeddedname);
 		gf_pso_sprintf (pso, "{1 index /FID ne {def} {pop pop} ifelse} forall\n");
 		gf_pso_sprintf (pso, "/Encoding [\n");
 		for (glyph = 0; glyph < 256; glyph++) {
@@ -1233,7 +1274,7 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
 			gf_pso_sprintf (pso, ((glyph & 0xf) == 0xf) ? "/%s\n" : "/%s ", c);
 		}
 		gf_pso_sprintf (pso, "] def currentdict end\n");
-		gf_pso_sprintf (pso, "/%s exch definefont pop\n", pso->encodedname);
+		gf_pso_sprintf (pso, "(%s) cvn exch definefont pop\n", pso->encodedname);
 	} else {
 		gint nfonts, nglyphs, i, j;
 		/* 16-bit vector */
@@ -1244,7 +1285,7 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
 		/* Common entries */
 		gf_pso_sprintf (pso, "/FontType 0 def\n");
 		gf_pso_sprintf (pso, "/FontMatrix [1 0 0 1 0 0] def\n");
-		gf_pso_sprintf (pso, "/FontName /%s-Glyph-Composite def\n", embeddedname);
+		gf_pso_sprintf (pso, "/FontName (%s-Glyph-Composite) cvn def\n", embeddedname);
 		gf_pso_sprintf (pso, "/LanguageLevel 2 def\n");
 
 		/* Type 0 entries */
@@ -1253,7 +1294,7 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
 		/* Bitch 'o' bitches */
 		gf_pso_sprintf (pso, "/FDepVector [\n");
 		for (i = 0; i < nfonts; i++) {
-			gf_pso_sprintf (pso, "/%s findfont dup length dict begin\n", embeddedname);
+			gf_pso_sprintf (pso, "(%s) cvn findfont dup length dict begin\n", embeddedname);
 			gf_pso_sprintf (pso, "{1 index /FID ne {def} {pop pop} ifelse} forall\n");
 			gf_pso_sprintf (pso, "/Encoding [\n");
 			for (j = 0; j < 256; j++) {
@@ -1271,7 +1312,7 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
 				gf_pso_sprintf (pso, ((j & 0xf) == 0xf) ? "/%s\n" : "/%s ", c);
 			}
 			gf_pso_sprintf (pso, "] def\n");
-			gf_pso_sprintf (pso, "currentdict end /%s-Glyph-Page-%d exch definefont\n", embeddedname, i);
+			gf_pso_sprintf (pso, "currentdict end (%s-Glyph-Page-%d) cvn exch definefont\n", embeddedname, i);
 		}
 		gf_pso_sprintf (pso, "] def\n");
 		gf_pso_sprintf (pso, "/Encoding [\n");
@@ -1282,7 +1323,7 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
 		}
 		gf_pso_sprintf (pso, "] def\n");
 		gf_pso_sprintf (pso, "currentdict end\n");
-		gf_pso_sprintf (pso, "/%s exch definefont pop\n", pso->encodedname);
+		gf_pso_sprintf (pso, "(%s) cvn exch definefont pop\n", pso->encodedname);
 	}
 }
 
@@ -1319,15 +1360,20 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 	embeddedname = pso->face->psname;
 
 	fbuf = b.buf;
-	strings = gp_tt_split_file (fbuf, b.buf_size);
+	strings = gp_tt_split_file (fbuf, b.buf_size, pso->face->entry->index);
 	if (strings) {
 		/* Relatively easy - just embed TTF into PS stream */
 		/* Download master font */
 		TTVersion = 1.0;
 		MfrRevision = 1.0;
-		gf_pso_sprintf (pso, "%%!PS-TrueTypeFont-%g-%g\n", TTVersion, MfrRevision);
+		gf_pso_sprintf (pso, "%%!PS-TrueTypeFont-");
+		gf_pso_print_double (pso, "%g", TTVersion);
+		gf_pso_sprintf (pso, "-");
+		gf_pso_print_double (pso, "%g", MfrRevision);
+		gf_pso_sprintf (pso, "\n");
+
 		gf_pso_sprintf (pso, "11 dict begin\n");
-		gf_pso_sprintf (pso, "/FontName /%s def\n", embeddedname);
+		gf_pso_sprintf (pso, "/FontName (%s) cvn def\n", embeddedname);
 		gf_pso_sprintf (pso, "/Encoding 256 array\n");
 		gf_pso_sprintf (pso, "0 1 255 {1 index exch /.notdef put} for\n");
 		gf_pso_sprintf (pso, "readonly def\n");
@@ -1335,7 +1381,15 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 		gf_pso_sprintf (pso, "/FontMatrix [1 0 0 1 0 0] def\n");
 		/* fixme: */
 		bbox = gnome_font_face_get_stdbbox (pso->face);
-		gf_pso_sprintf (pso, "/FontBBox [%g %g %g %g] def\n", bbox->x0, bbox->y0, bbox->x1, bbox->y1);
+		gf_pso_sprintf (pso, "/FontBBox [");
+		gf_pso_print_double (pso, "%g",  bbox->x0);
+		gf_pso_sprintf (pso, " ");
+		gf_pso_print_double (pso, "%g", bbox->y0);
+		gf_pso_sprintf (pso, " ");
+		gf_pso_print_double (pso, "%g",  bbox->x1);
+		gf_pso_sprintf (pso, " ");
+		gf_pso_print_double (pso, "%g",  bbox->y1);
+		gf_pso_sprintf (pso, "] def\n");
 		gf_pso_sprintf (pso, "/FontType 42 def\n");
 		/* fixme: XUID */
 		/* fixme: Be more intelligent */
@@ -1403,7 +1457,7 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 		gint glyph;
 		/* 8-bit vector */
 		pso->encodedbytes = 1;
-		gf_pso_sprintf (pso, "/%s findfont dup length dict begin\n", embeddedname);
+		gf_pso_sprintf (pso, "(%s) cvn findfont dup length dict begin\n", embeddedname);
 		gf_pso_sprintf (pso, "{1 index /FID ne {def} {pop pop} ifelse} forall\n");
 		gf_pso_sprintf (pso, "/Encoding [\n");
 		for (glyph = 0; glyph < 256; glyph++) {
@@ -1416,7 +1470,7 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 			}
 		}
 		gf_pso_sprintf (pso, "] def currentdict end\n");
-		gf_pso_sprintf (pso, "/%s exch definefont pop\n", pso->encodedname);
+		gf_pso_sprintf (pso, "(%s) cvn exch definefont pop\n", pso->encodedname);
 	} else {
 		gint nfonts, nglyphs, i, j;
 		/* 16-bit vector */
@@ -1428,14 +1482,14 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 		/* Common entries */
 		gf_pso_sprintf (pso, "/FontType 0 def\n");
 		gf_pso_sprintf (pso, "/FontMatrix [1 0 0 1 0 0] def\n");
-		gf_pso_sprintf (pso, "/FontName /%s-Glyph-Composite def\n", embeddedname);
+		gf_pso_sprintf (pso, "/FontName (%s-Glyph-Composite) cvn def\n", embeddedname);
 		gf_pso_sprintf (pso, "/LanguageLevel 2 def\n");
 		/* Type 0 entries */
 		gf_pso_sprintf (pso, "/FMapType 2 def\n");
 		/* Bitch 'o' bitches */
 		gf_pso_sprintf (pso, "/FDepVector [\n");
 		for (i = 0; i < nfonts; i++) {
-			gf_pso_sprintf (pso, "/%s findfont dup length dict begin\n", embeddedname);
+			gf_pso_sprintf (pso, "(%s) cvn findfont dup length dict begin\n", embeddedname);
 			gf_pso_sprintf (pso, "{1 index /FID ne {def} {pop pop} ifelse} forall\n");
 			gf_pso_sprintf (pso, "/Encoding [\n");
 			for (j = 0; j < 256; j++) {
@@ -1450,7 +1504,7 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 				}
 			}
 			gf_pso_sprintf (pso, "] def\n");
-			gf_pso_sprintf (pso, "currentdict end /%s-Glyph-Page-%d exch definefont\n", embeddedname, i);
+			gf_pso_sprintf (pso, "currentdict end (%s-Glyph-Page-%d) cvn exch definefont\n", embeddedname, i);
 		}
 		gf_pso_sprintf (pso, "] def\n");
 		gf_pso_sprintf (pso, "/Encoding [\n");
@@ -1461,7 +1515,7 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 		}
 		gf_pso_sprintf (pso, "] def\n");
 		gf_pso_sprintf (pso, "currentdict end\n");
-		gf_pso_sprintf (pso, "/%s exch definefont pop\n", pso->encodedname);
+		gf_pso_sprintf (pso, "(%s) cvn exch definefont pop\n", pso->encodedname);
 	}
 }
 
@@ -1489,16 +1543,16 @@ gnome_font_face_ps_embed_empty (GnomeFontPsObject *pso)
 	gf_pso_sprintf (pso, "/BuildChar {1 index /Encoding get exch get 1 index /BuildGlyph get exec } bind def\n");
 	if (pso->encodedbytes == 1) {
 		/* 8-bit empty font */
-		gf_pso_sprintf (pso, "currentdict end /%s exch definefont pop\n", pso->encodedname);
+		gf_pso_sprintf (pso, "currentdict end (%s) cvn exch definefont pop\n", pso->encodedname);
 	} else {
 		/* 16-bit empty font */
-		gf_pso_sprintf (pso, "currentdict end /%s-Base exch definefont pop\n", pso->encodedname);
+		gf_pso_sprintf (pso, "currentdict end (%s-Base) cvn exch definefont pop\n", pso->encodedname);
 		gf_pso_sprintf (pso, "32 dict begin /FontType 0 def /FontMatrix [1 0 0 1 0 0] def\n");
-		gf_pso_sprintf (pso, "/FontName /%s-Glyph-Composite def\n", pso->encodedname);
+		gf_pso_sprintf (pso, "/FontName (%s-Glyph-Composite) cvn def\n", pso->encodedname);
 		gf_pso_sprintf (pso, "/LanguageLevel 2 def /FMapType 2 def\n");
-		gf_pso_sprintf (pso, "/FDepVector [/%s-Base findfont] def", pso->encodedname);
+		gf_pso_sprintf (pso, "/FDepVector [(%s-Base) cvn findfont] def", pso->encodedname);
 		gf_pso_sprintf (pso, "/Encoding 256 array def 0 1 255 {Encoding exch 0 put} for\n");
-		gf_pso_sprintf (pso, "currentdict end /%s exch definefont pop\n", pso->encodedname);
+		gf_pso_sprintf (pso, "currentdict end (%s) cvn exch definefont pop\n", pso->encodedname);
 	}
 }
 
@@ -1510,29 +1564,38 @@ gf_pso_print_sized (GnomeFontPsObject *pso, const guchar *text, gint size)
 	pso->length += size;
 }
 
+/* Note "format" should be locale independent, so it should not use %g */
+/* and friends */
 static void
 gf_pso_sprintf (GnomeFontPsObject *pso, const gchar * format, ...)
 {
 	va_list arguments;
-	gchar *oldlocale;
 	gchar *text;
-	gint len;
 	
-	oldlocale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
-		
 	va_start (arguments, format);
 	text = g_strdup_vprintf (format, arguments);
 	va_end (arguments);
 
-	len = strlen (text);
-	gf_pso_print_sized (pso, text, len);
+	gf_pso_print_sized (pso, text, strlen (text));
+	g_free (text);
+}
+
+/* Allowed conversion specifiers are 'e', 'E', 'f', 'F', 'g' and 'G'. */
+static gint   
+gf_pso_print_double (GnomeFontPsObject *pso, const gchar *format, gdouble x)
+{
+ 	gchar *text;
+
+	text = g_new (gchar, G_ASCII_DTOSTR_BUF_SIZE);
+	g_ascii_formatd (text, G_ASCII_DTOSTR_BUF_SIZE, format, x);
+
+	gf_pso_print_sized (pso, text, strlen (text));
 	g_free (text);
 
-	setlocale (LC_NUMERIC, oldlocale);
-	g_free (oldlocale);
+	return GNOME_PRINT_OK;
 }
 
+
 static void
 gnome_font_face_ps_embed_ensure_size (GnomeFontPsObject *pso, gint size)
 {
