@@ -182,8 +182,9 @@ gnome_font_get_prop (GObject *o, guint id, GValue *value, GParamSpec *pspec)
 	case PROP_FAMILYNAME:
 		g_value_set_string (value, gnome_font_face_get_family_name (font->face));
 		break;
-	case PROP_WEIGHT:
-		g_value_set_string (value, font->face->entry->weight);
+	case PROP_WEIGHT:                      
+                /* FIXME: we  should be using GnomeFontWeight */
+		g_value_set_string (value, font->face->entry->speciesname);
 		break;
 	case PROP_ITALICANGLE:
 		/* fixme: implement */
@@ -285,19 +286,11 @@ gnome_font_find_closest (const guchar *name, gdouble size)
 	GnomeFontFace *face;
 	GnomeFont *font;
 
-	if (name)
-		face = gnome_font_face_find (name);
-	else
-		face = NULL;
+	face = gnome_font_face_find (name);
 
-	if (!face) {
-		/* fixme: */
-		face = gnome_font_face_find_closest ("Sans Regular");
-		g_return_val_if_fail (face != NULL, NULL);
-	}
+	g_return_val_if_fail (face != NULL, NULL);
 
 	font = gnome_font_face_get_font_full (face, size, NULL);
-
 	gnome_font_face_unref (face);
 
 	return font;
