@@ -608,11 +608,20 @@ gnome_print_pdf_glyphlist (GnomePrintContext *pc, const gdouble *ctm, GnomeGlyph
 
 		memcpy (Tm, ctm, 6 * sizeof (gdouble));
 		art_affine_scale (Tm, font->size, font->size);
-		gnome_print_pdf_page_fprintf (pdf,
-					      "%f %f %f %f %f %f Tm" EOL,
-					      Tm [0], Tm [1], Tm [2], Tm [3],
-					      dx + pgl->glyphs[ps->start].x,
-					      dy + pgl->glyphs[ps->start].y);
+		
+		gnome_print_pdf_page_print_double (pdf, "%g", Tm [0]);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%g", Tm [1]);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%g", Tm [2]);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%g", Tm [3]);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%g", dx + pgl->glyphs[ps->start].x);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%g", dy + pgl->glyphs[ps->start].y);
+		gnome_print_pdf_page_fprintf (pdf, " Tm" EOL);
+
 		/* Build string */
 		ret = gnome_print_pdf_page_fprintf (pdf, "(");
 		
